/*
** Filename: android_acc.c
**
** Automatically created by Application Wizard 1.4.2
**
** Part of solution android_acc in project android_acc
**
** Comments:
**
** Important: Sections between markers "FTDI:S*" and "FTDI:E*" will be overwritten by
** the Application Wizard
*/

#include "android_acc.h"

/* FTDI:STP Thread Prototypes */
vos_tcb_t *tcbusbFIRMWARE;
vos_tcb_t *tcbpwmFIRMWARE;
vos_tcb_t *tcbgpioFIRMWARE;

void firmware();
void open_drivers(void);
void usb_host_processing();
void pwm_processing();

/* FTDI:SDH Driver Handles */
VOS_HANDLE hUSBHOST_1; // USB Host Port 1
VOS_HANDLE hANDROID_ACCESSORY; // Android Open Accessory Class Driver
VOS_HANDLE hGPIO_PORT_A; // GPIO Port A Driver
VOS_HANDLE hGPIO_PORT_B; // GPIO Port B Driver
VOS_HANDLE hPWM; // PWM Driver
/* FTDI:EDH */


/*global variables*/
#define PWM_MAX_DUTY_CYCLE		45
#define PWM_MIN_DUTY_CYCLE		3
#define PWM_MAX_DUTY_COUNT		50

unsigned char u8PwmDutyCycle = 0;
unsigned char u8Changed = 0;
unsigned char u8AccesoryConnected = 0;




/*accessory packet*/
android_accessory_packet gstAccPacketWrite;
android_accessory_packet gstAccPacketRead;



/* Declaration for IOMUx setup function */
void iomux_setup(void);

/* Main code - entry point to firmware */
void main(void)
{
	/* FTDI:SDD Driver Declarations */
	// GPIO Port A configuration context
	gpio_context_t gpioContextA;
	// GPIO Port B configuration context
	gpio_context_t gpioContextB;
	// USB Host configuration context
	usbhost_context_t usbhostContext;
	/* FTDI:EDD */

	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(512);
	/* FTDI:EKI */

	iomux_setup();

	/* FTDI:SDI Driver Initialisation */
	// Initialise GPIO A
	gpioContextA.port_identifier = GPIO_PORT_A;
	gpio_init(VOS_DEV_GPIO_PORT_A,&gpioContextA);

	// Initialise GPIO B
	gpioContextB.port_identifier = GPIO_PORT_B;
	gpio_init(VOS_DEV_GPIO_PORT_B,&gpioContextB);

	// Initialise PWM
	pwm_init(VOS_DEV_PWM);

	// Initialise USB Host
	usbhostContext.if_count = 8;
	usbhostContext.ep_count = 16;
	usbhostContext.xfer_count = 2;
	usbhostContext.iso_xfer_count = 2;
	usbhost_init(VOS_DEV_USBHOST_1, -1, &usbhostContext);
	/* FTDI:EDI */
	// init Android Accessory
	usbHostAndroidAccessory_init(VOS_DEV_ANDROID_ACCESSORY);


	/* FTDI:SCT Thread Creation */
	tcbusbFIRMWARE = vos_create_thread_ex(20, 2048, usb_host_processing, "usbApplication", 0);
	tcbgpioFIRMWARE = vos_create_thread_ex(20, 1024, firmware, "gpioApplication", 0);
	tcbpwmFIRMWARE = vos_create_thread_ex(20, 1024, pwm_processing, "pwmApplication", 0);
	/* FTDI:ECT */


	/*open the drivers*/
	 open_drivers();

	/*enable the PWM interrupts*/
	vos_enable_interrupts(VOS_PWM_TOP_INT_IEN);

	vos_start_scheduler();

main_loop:
	goto main_loop;

}

/* FTDI:SSP Support Functions */

unsigned char usbhost_connect_state(VOS_HANDLE hUSB)
{
	unsigned char connectstate = PORT_STATE_DISCONNECTED;
	usbhost_ioctl_cb_t hc_iocb;

	if (hUSB)
	{
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_GET_CONNECT_STATE;
		hc_iocb.get        = &connectstate;
		vos_dev_ioctl(hUSB, &hc_iocb);
	}
	return connectstate;
}


VOS_HANDLE android_attach(VOS_HANDLE hUSB, unsigned char devANDACC,
	char *manufacturer, char *model, char *description,
	char *version, char *uri, char *serial)
{
	common_ioctl_cb_t androidAccessory_cb;
	usbHostAndroidAccessory_ioctl_cb_attach_t atInfo;
	VOS_HANDLE hANDACC;

	hANDACC = vos_dev_open(devANDACC);

	atInfo.hc_handle = hUSB;
	atInfo.manufacturer = manufacturer;
	atInfo.model = model;
	atInfo.description = description;
	atInfo.version = version;
	atInfo.uri = uri;
	atInfo.serial = serial;

	androidAccessory_cb.ioctl_code = VOS_IOCTL_USBHOSTANDROIDACCESSORY_ATTACH;
	androidAccessory_cb.set.data = &atInfo;
	if (vos_dev_ioctl(hANDACC, &androidAccessory_cb) != USBHOSTANDROIDACCESSORY_OK)
	{
		vos_dev_close(hANDACC);
		hANDACC = NULL;
	}

	return hANDACC;
}

void android_detach(VOS_HANDLE hANDACC)
{
	common_ioctl_cb_t androidAccessory_cb;

	if (hANDACC)
	{
		androidAccessory_cb.ioctl_code = VOS_IOCTL_USBHOSTANDROIDACCESSORY_DETACH;
		vos_dev_ioctl(hANDACC, &androidAccessory_cb);
		vos_dev_close(hANDACC);
	}
}

/* FTDI:ESP */

void open_drivers(void)
{
      /* Code for opening and closing drivers - move to required places in Application Threads */
        /* FTDI:SDA Driver Open */
        hUSBHOST_1 = vos_dev_open(VOS_DEV_USBHOST_1);
        hGPIO_PORT_A = vos_dev_open(VOS_DEV_GPIO_PORT_A);
        hGPIO_PORT_B = vos_dev_open(VOS_DEV_GPIO_PORT_B);
        hPWM = vos_dev_open(VOS_DEV_PWM);
		//hANDROID_ACCESSORY = vos_dev_open(VOS_DEV_ANDROID_ACCESSORY);
}

void attach_drivers(void)
{
        /* FTDI:SUA Layered Driver Attach Function Calls */
        // Android Accessory Driver attach must specify various strings
        // Suggested values provided
        //char *manufacturer = "FTDI\0";
        //char *model = "VNC2\0";
        //char *description = "Vinculum Accessory Test\0";
        //char *version = "0.1.0\0";
        //char *uri = "http://www.ftdichip.com\0";
        //char *serial = "VinculumAccessory1\0";
        //hANDROID_ACCESSORY = android_attach(hUSBHOST_1, VOS_DEV_ANDROID_ACCESSORY,
        // manufacturer, model, description, version, uri, serial);
        /* FTDI:EUA */
}

void close_drivers(void)
{
        /* FTDI:SDB Driver Close */
        vos_dev_close(hUSBHOST_1);
        vos_dev_close(hGPIO_PORT_A);
        vos_dev_close(hGPIO_PORT_B);
        vos_dev_close(hPWM);
		//vos_dev_close(hANDROID_ACCESSORY);
        /* FTDI:EDB */
}

/* Application Threads */

#define PUSH_BUTTON1_MAP 	(0x02) //A1
#define PUSH_BUTTON2_MAP 	(0x04) //A2
#define PUSH_BUTTON3_MAP  	(0x08) //A3
#define PUSH_BUTTON4_MAP	(0x10)  //A4
#define PUSH_BUTTON5_MAP	(0x08)  //Not Connected

#define LED_BUTTON_MAP	   	(0x78)
#define PUSH_BUTTON_MAP 	(0x1E)


/*FIXME, this fix si done to clear the issue of
				  wrong port read
*/
unsigned char u8PrevLedMap = 0x78;
/*to handle the long press*/
unsigned char u8PrevKeyMap = 0x1E;
/*temporary variable*/
unsigned char u8KeyMap = 0x1E;

/*since there is only one timer, I dont know how its going to
  workout
*/
void usb_host_processing()
{

	unsigned char i;
	unsigned char numRead = 0;
	usbhost_ioctl_cb_t                         usbhost_iocb;
	common_ioctl_cb_t                          androidAccessory_cb;
	usbHostAndroidAccessory_ioctl_cb_attach_t  atInfo;
	unsigned char                              status;

	// setup strings for the accessory
	// the manufacturer, model and version strings should match those in the
	// application's accessory_filter.xml file!
	// This will allow the application to auto-launch when the accessory is
	// connected
	char *manufacturer = "FTDI\0";
	char *model = "FTDIDemoKit\0";
	char *description = "Vinculum Accessory Test\0";
	char *version = "1.0\0";
	char *uri = "http://www.ftdichip.com\0";
	char *serial = "VinculumAccessory1\0";
	uint8 u8Data;
	unsigned char ledState = 0;
	//gpio_ioctl_cb_t gpio_iocb;
	unsigned char portData = 0xFF; // LEDs on when low, off when high
	unsigned short protocolRevision = 0;


	while (1)
	{
		if(usbhost_connect_state(hUSBHOST_1)== PORT_STATE_ENUMERATED)
		{

			hANDROID_ACCESSORY = android_attach(hUSBHOST_1, VOS_DEV_ANDROID_ACCESSORY,
												manufacturer, model, description, version, uri, serial);

			if (hANDROID_ACCESSORY)
			{

					// successfully found and attached to an Android Accessory device
					// Write data to the GPIO port - bit 3 to toggle LED1 on V2EVAL board.
					vos_dev_write(hGPIO_PORT_B,&u8PrevLedMap, 1, NULL);
					// get the protocol revision - should be 0x0100 for now
					androidAccessory_cb.ioctl_code = VOS_IOCTL_USBHOSTANDROIDACCESSORY_GET_PROTOCOL_REVISION;
					androidAccessory_cb.get.data = &protocolRevision;
					status = vos_dev_ioctl(hANDROID_ACCESSORY, &androidAccessory_cb);
					if (protocolRevision != 0x100)
					{
						vos_halt_cpu();
					}

					while (status == USBHOSTANDROIDACCESSORY_OK)
					{
						/*we are connected*/
						u8AccesoryConnected = 1;
						// read a message from the Android device
						// wrap this in the accessory driver read function
						// NOTE: this call may return with less data than was requested
						// In this case, we are requesting 64 bytes to fill our buffer,
						// but our Android app will only send 1 byte!
						status = vos_dev_read(hANDROID_ACCESSORY,(uint8 *)&gstAccPacketRead, sizeof(gstAccPacketRead) , &numRead);

					  if(numRead == sizeof(gstAccPacketRead))
						{
						// process the message from the Android device
						if(gstAccPacketRead.u8Type == DATA_TYPE_KEYPAD)
						{
							/*update the led bitmap*/
							u8Data = (gstAccPacketRead.u8Data << 3);
							u8Data &= LED_BUTTON_MAP;
							u8PrevLedMap ^= u8Data;
							vos_dev_write(hGPIO_PORT_B,&u8PrevLedMap, 1, NULL);
						}
						else if(gstAccPacketRead.u8Type == DATA_TYPE_SLIDER)
						{
							u8Changed = 1;
							u8PwmDutyCycle = gstAccPacketRead.u8Data;

						}
					  }
					}/*while(status == USBHOSTANDROIDACCESSORY_OK)*/
					/*we are disconnected*/
					u8AccesoryConnected = 0;
					/*detach accessory*/
					android_detach(hANDROID_ACCESSORY);

					u8PrevLedMap = LED_BUTTON_MAP;
					vos_dev_write(hGPIO_PORT_B, &u8PrevLedMap, 1, NULL);

					/*off the pwm led*/
					u8Changed = 1;
					u8PwmDutyCycle = 0x00;


					/*wait unitl its uninstalled*/
					while(usbhost_connect_state(hUSBHOST_1)== PORT_STATE_ENUMERATED)
					{
						vos_delay_msecs(10);
  					}
				} /*if (hANDROID_ACCESSORY)*/
		}/*if(usbhost_connect_state(VOS_HANDLE hUSB)== PORT_STATE_ENUMERATED)*/
 	 } /*end of while 1*/
	/*wait for sometime*/
	vos_delay_msecs(10);
}




/* Application Threads */

void firmware()
{
	/* Thread code to be added here */
	unsigned char u8PortA = 0xF0;
	unsigned char u8PortB;
	unsigned char u8Temp;

	gpio_ioctl_cb_t gpio_ioca;

    // Set all pins to output using an ioctl.
	gpio_ioca.ioctl_code = VOS_IOCTL_GPIO_SET_MASK;
	gpio_ioca.value      = 0x78;
	// Send the ioctl to the device manager.
	vos_dev_ioctl(hGPIO_PORT_B, &gpio_ioca);
	/*clear the LEDs*/
    vos_dev_write(hGPIO_PORT_B,&u8PrevLedMap, 1, NULL);

	/*initialize the accessory, packet, for now
	  it supports only leds*/
	gstAccPacketWrite.u8Type = DATA_TYPE_KEYPAD;
	while(1)
	{
		//svos_dev_write(hGPIO_PORT_A, &u8PortA, 1, NULL);
		/*check the key press*/
		vos_dev_read(hGPIO_PORT_A,&u8PortA, 1, NULL);
		/*take the one's cmplement*/
		u8PortA = ~u8PortA;
		/*the high ones are pressed*/
		u8PortA &= PUSH_BUTTON_MAP;

		/*check whether the key has been held*/
		if(u8PortA == u8PrevKeyMap)
		        continue;
		else
			u8PrevKeyMap = u8PortA;

		/*if any button pressed??*/
		if(u8PortA)
		{
		/*look for new key*/
		//u8PrevKeyMap = u8PortA;
		//u8ValidKey = 1;



			/*start the debounce of 200 msess, too much though*/
			//wait_debounce(100);
			vos_delay_msecs(100);
			vos_dev_read(hGPIO_PORT_A, &u8Temp, 1, NULL);

			u8Temp = ~u8Temp;
			u8Temp &= PUSH_BUTTON_MAP;

			u8PortA &= u8Temp;

			/*check if any of the keys are presseed*/
			if(u8PortA){
/*
				u8Changed = 1;
				if(u8PortA & PUSH_BUTTON1_MAP){
					u8PwmDutyCycle++;
				 }else if(u8PortA & PUSH_BUTTON2_MAP){
						if(u8PwmDutyCycle !=0){
					 		u8PwmDutyCycle--;
						}
			     }else if(u8PortA & PUSH_BUTTON3_MAP){
					u8PwmDutyCycle = 50;
				}
*/
				/*led on positions*/
				u8PortA <<= 2;
				/*read the port A again to check which leds are high*/
				//vos_dev_read(hGPIO_PORT_A, &u8Temp, 1, NULL);
				//u8Temp1 &= 0xE0;
				/*just take the map of LEDs only*/
				/*FIXME, this fix si done to clear the issue of
				  wrong port read
				*/
				u8PrevLedMap ^= u8PortA;
				vos_dev_write(hGPIO_PORT_B,&u8PrevLedMap, 1, NULL);

				/*prepare the packet*/
				gstAccPacketWrite.u8Data = (u8PortA >> 3);

				if(u8AccesoryConnected == 1){
				/*send the usb data accross*/
				vos_dev_write(hANDROID_ACCESSORY,(uint8 *)&gstAccPacketWrite, sizeof(gstAccPacketWrite), NULL);
			}

			}

		}


 } /*end of while */
}/*end of firmware*/


void pwm_processing()
{
      pwm_ioctl_cb_t pwm_iocb;
	  uint16 u16OnTime = 500;

	  // set counter prescaler value
      pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_PRESCALER_VALUE;
 	  pwm_iocb.count.prescaler = 0x01;
      vos_dev_ioctl(hPWM, &pwm_iocb);

	  /*FIXME, keep it like this, till I find a better way
	    to enable/disable pwm
	  */


	while(1) {

		 /*max is 50*/
		if(u8PwmDutyCycle > PWM_MAX_DUTY_COUNT){
				pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_INITIAL_STATE;
		 		pwm_iocb.output.init_state_mask = 0x00;
	  			vos_dev_ioctl(hPWM, &pwm_iocb);
				vos_delay_msecs(10);
				continue;
			}

		/*take care of minimum duty cycle*/
		//if(u8PwmDutyCycle < PWM_MIN_DUTY_CYCLE){
		//		u8PwmDutyCycle = PWM_MIN_DUTY_CYCLE;

	   //}

	    /*calculate the ON time*/
 	   /*take care of the overflow*/
	  if(u8PwmDutyCycle < PWM_MIN_DUTY_CYCLE){
			pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_INITIAL_STATE;
		 	pwm_iocb.output.init_state_mask = 0xff;
	  		vos_dev_ioctl(hPWM, &pwm_iocb);
			vos_delay_msecs(10);
			continue;
		}else{
	   		u16OnTime = (1000*u8PwmDutyCycle)/PWM_MAX_DUTY_COUNT;
	   		u16OnTime = (u16OnTime*48);
	  }
        // *******************************************************************

         // Setting a count value of 0x00A0 with toggles at 0x0010 and 0x0060

         // will give a 50% duty cycle

         // *******************************************************************



         // set counter value - cycle complete when internal counter reaches this value

         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COUNTER_VALUE;
         pwm_iocb.count.value = 0xbb80;/*for 1 khz*/
         vos_dev_ioctl(hPWM, &pwm_iocb);



         // set comparator 0 value - toggle output at a value of 0x0010

         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
         pwm_iocb.identifier.comparator_number = COMPARATOR_0;
         pwm_iocb.comparator.value = u16OnTime;//0x0960; /*50 % duty cycle*/
         vos_dev_ioctl(hPWM, &pwm_iocb);



         // set comparator 1 value - toggle output at a value of 0x0060

         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
         pwm_iocb.identifier.comparator_number = COMPARATOR_1;
         pwm_iocb.comparator.value = 0xbb80;//
         vos_dev_ioctl(hPWM, &pwm_iocb);



         // enable comparators 0 and 1 for PWM 3

         // this will cause PWM output 1 to toggle on comparators 0 and 1

         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
         pwm_iocb.identifier.pwm_number = PWM_3;
         pwm_iocb.output.enable_mask = (MASK_COMPARATOR_0 | MASK_COMPARATOR_1);
         vos_dev_ioctl(hPWM, &pwm_iocb);



         // set initial state - all PWM outputs will be low (0) initially

         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_INITIAL_STATE;
		 if(u8PwmDutyCycle < PWM_MIN_DUTY_CYCLE){
         	pwm_iocb.output.init_state_mask = 0xff;
	  	}else{
		 	pwm_iocb.output.init_state_mask = 0x00;
		}
         vos_dev_ioctl(hPWM, &pwm_iocb);



         // set restore state - PWM output 3 will return to low state (0)

         // at end of cycle
         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_RESTORE_INITIAL_STATE;
         pwm_iocb.output.restore_state_mask = (MASK_PWM_0);
         vos_dev_ioctl(hPWM, &pwm_iocb);


         // set mode to 25 cycles

         pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_NUMBER_OF_CYCLES;
         pwm_iocb.output.mode = 0x50;
         vos_dev_ioctl(hPWM, &pwm_iocb);
         while(1)
		  	{

                 // enable interrupt - this will fire when the specified number of

                 // cycles is complete

                 pwm_iocb.ioctl_code = VOS_IOCTL_PWM_ENABLE_INTERRUPT;
                 vos_dev_ioctl(hPWM, &pwm_iocb);


					// enable output
			//	if(u8PwmDutyCycle > PWM_MIN_DUTY_CYCLE){
                 pwm_iocb.ioctl_code = VOS_IOCTL_PWM_ENABLE_OUTPUT;
                 vos_dev_ioctl(hPWM, &pwm_iocb);
			 // }

                 // wait on interrupt
                 pwm_iocb.ioctl_code = VOS_IOCTL_PWM_WAIT_ON_COMPLETE;
                 vos_dev_ioctl(hPWM, &pwm_iocb);

				 // disable output
                 pwm_iocb.ioctl_code = VOS_IOCTL_PWM_DISABLE_OUTPUT;
				 vos_dev_ioctl(hPWM, &pwm_iocb);


				if(u8Changed == 1){
					 	u8Changed = 0;
				//		/*goto the external loop*/
						break;
				 }

			  }

        }
}
